
/*
 *
 * This code is a reproduction of standard TI code


 * Copyright (c) 2020, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <lab4_helper.h>

void delay_cycles(uint32_t cycles)
{
    /* this is a scratch register for the compiler to use */
    uint32_t scratch;

    /* There will be a 2 cycle delay here to fetch & decode instructions
     * if branch and linking to this function */

    /* Subtract 2 net cycles for constant offset: +2 cycles for entry jump,
     * +2 cycles for exit, -1 cycle for a shorter loop cycle on the last loop,
     * -1 for this instruction */

    __asm volatile(
#ifdef __GNUC__
        ".syntax unified\n\t"
#endif
        "SUBS %0, %[numCycles], #2; \n"
        "%=: \n\t"
        "SUBS %0, %0, #4; \n\t"
        "NOP; \n\t"
        "BHS  %=b;" /* branches back to the label defined above if number > 0 */
        /* Return: 2 cycles */
        : "=&r"(scratch)
        : [ numCycles ] "r"(cycles));
}


void InitializeSleepIsStandby(void) {
    SYSCTL->SOCLOCK.PMODECFG = SYSCTL_PMODECFG_DSLEEP_STANDBY;
    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
    SYSCTL->SOCLOCK.MCLKCFG &= ~(SYSCTL_MCLKCFG_STOPCLKSTBY_ENABLE);
}

void InitializeProcessor(void) {
    SYSCTL->SOCLOCK.BORTHRESHOLD = SYSCTL_SYSSTATUS_BORCURTHRESHOLD_BORMIN; // Brownout generates a reset.

    update_reg(&SYSCTL->SOCLOCK.MCLKCFG, (uint32_t) SYSCTL_MCLKCFG_UDIV_NODIVIDE, SYSCTL_MCLKCFG_UDIV_MASK); // Set UPCLK divider
    update_reg(&SYSCTL->SOCLOCK.SYSOSCCFG, SYSCTL_SYSOSCCFG_FREQ_SYSOSCBASE, SYSCTL_SYSOSCCFG_FREQ_MASK); // Set SYSOSC to 32 MHz

    // Disable MCLK Divider
    update_reg(&SYSCTL->SOCLOCK.MCLKCFG, (uint32_t) 0x0, SYSCTL_MCLKCFG_MDIV_MASK);
}


void InitializeTimerG0(void) {

    TIMG0->GPRCM.RSTCTL = (GPIO_RSTCTL_KEY_UNLOCK_W | GPIO_RSTCTL_RESETSTKYCLR_CLR | GPIO_RSTCTL_RESETASSERT_ASSERT);
    TIMG0->GPRCM.PWREN = (GPIO_PWREN_KEY_UNLOCK_W | GPIO_PWREN_ENABLE_ENABLE);
    delay_cycles(POWER_STARTUP_DELAY); // delay to enable GPIO to turn on and reset

    // Configure clocking for Timer Module
    TIMG0->CLKSEL = GPTIMER_CLKSEL_LFCLK_SEL_ENABLE;
    TIMG0->CLKDIV = GPTIMER_CLKDIV_RATIO_DIV_BY_1;

    /* This will configure what happens when we count down to zero - we'll set counter to LOAD value */

    TIMG0->COUNTERREGS.CC_01[0] = 0; // Count down to zero (value in CC0), then reload
    TIMG0->COUNTERREGS.CCCTL_01[0] = GPTIMER_CCCTL_01_ACOND_TIMCLK; // Set timer to advance on TIMCLK ticks

    // CM_DOWN - count down mode; REPEAT_1 - keep going when we reach zero; CVAE_LDVAL - when we get to zero, reload LOAD value;
    // EN_DISABLED - Start with timer disabled
    TIMG0->COUNTERREGS.CTRCTL =
        (( GPTIMER_CTRCTL_CVAE_LDVAL | GPTIMER_CTRCTL_CM_DOWN | GPTIMER_CTRCTL_REPEAT_REPEAT_1) | GPTIMER_CTRCTL_EN_DISABLED);

    // Enable timer interrupt when we reach 0
    TIMG0->CPU_INT.IMASK |= GPTIMER_CPU_INT_IMASK_Z_SET;

    TIMG0->PDBGCTL = GPTIMER_PDBGCTL_SOFT_IMMEDIATE;

    TIMG0->COMMONREGS.CCLKCTL = (GPTIMER_CCLKCTL_CLKEN_ENABLED);

}


void InitializeTimerA1_G8_PWM(void) {
    TIMA1->GPRCM.RSTCTL = (GPIO_RSTCTL_KEY_UNLOCK_W | GPIO_RSTCTL_RESETSTKYCLR_CLR | GPIO_RSTCTL_RESETASSERT_ASSERT);
    TIMA1->GPRCM.PWREN = (GPIO_PWREN_KEY_UNLOCK_W | GPIO_PWREN_ENABLE_ENABLE);
    delay_cycles(POWER_STARTUP_DELAY); // delay to enable GPIO to turn on and reset


    // Configure clocking for Timer Module
//    TIMA1->CLKSEL = GPTIMER_CLKSEL_BUSCLK_SEL_ENABLE;
    TIMA1->CLKSEL = GPTIMER_CLKSEL_LFCLK_SEL_ENABLE;
    TIMA1->CLKDIV = GPTIMER_CLKDIV_RATIO_DIV_BY_1;
//    TIMA1->COMMONREGS.CPS = 25; // Clock prescaling - 8 bits

    TIMA1->COUNTERREGS.LOAD = 1999; // Count up max. Period will be 2000

    TIMA1->COUNTERREGS.CCACT_01[0] = GPTIMER_CCACT_01_ZACT_CCP_HIGH | GPTIMER_CCACT_01_CUACT_CCP_LOW;
    TIMA1->COUNTERREGS.CCACT_01[1] = GPTIMER_CCACT_01_ZACT_CCP_HIGH | GPTIMER_CCACT_01_CUACT_CCP_LOW;
    TIMA1->COUNTERREGS.CTRCTL = GPTIMER_CTRCTL_REPEAT_REPEAT_1 | GPTIMER_CTRCTL_CM_UP |
            GPTIMER_CTRCTL_CVAE_ZEROVAL | GPTIMER_CTRCTL_EN_DISABLED;
    TIMA1->COMMONREGS.CCPD = GPTIMER_CCPD_C0CCP0_OUTPUT | GPTIMER_CCPD_C0CCP1_OUTPUT;
    TIMA1->COMMONREGS.CCLKCTL = (GPTIMER_CCLKCTL_CLKEN_ENABLED);

    TIMA1->COUNTERREGS.CC_01[0] = 0; // initial value
    TIMA1->COUNTERREGS.CC_01[1] = 0;
    TIMA1->COUNTERREGS.CTRCTL |= (GPTIMER_CTRCTL_EN_ENABLED);



    TIMG8->GPRCM.RSTCTL = (GPIO_RSTCTL_KEY_UNLOCK_W | GPIO_RSTCTL_RESETSTKYCLR_CLR | GPIO_RSTCTL_RESETASSERT_ASSERT);
    TIMG8->GPRCM.PWREN = (GPIO_PWREN_KEY_UNLOCK_W | GPIO_PWREN_ENABLE_ENABLE);
    delay_cycles(POWER_STARTUP_DELAY); // delay to enable GPIO to turn on and reset

    // Configure clocking for Timer Module
    TIMG8->CLKSEL = GPTIMER_CLKSEL_LFCLK_SEL_ENABLE;
    TIMG8->CLKDIV = GPTIMER_CLKDIV_RATIO_DIV_BY_1;
//    TIMG8->COMMONREGS.CPS = 25; // Clock prescaling (what is this???)

    TIMG8->COUNTERREGS.LOAD = 1999;

    TIMG8->COUNTERREGS.CCACT_01[1] = GPTIMER_CCACT_01_ZACT_CCP_HIGH | GPTIMER_CCACT_01_CUACT_CCP_LOW;
    TIMG8->COUNTERREGS.CTRCTL = GPTIMER_CTRCTL_REPEAT_REPEAT_1 | GPTIMER_CTRCTL_CM_UP |
            GPTIMER_CTRCTL_CVAE_ZEROVAL | GPTIMER_CTRCTL_EN_DISABLED;
    TIMG8->COMMONREGS.CCPD = GPTIMER_CCPD_C0CCP1_OUTPUT;
    TIMG8->COMMONREGS.CCLKCTL = (GPTIMER_CCLKCTL_CLKEN_ENABLED);

    TIMG8->COUNTERREGS.CC_01[1] = 0;
    TIMG8->COUNTERREGS.CTRCTL |= (GPTIMER_CTRCTL_EN_ENABLED);

    // CC0 action
//    update_reg(&TIMA1->COUNTERREGS.CCACT_01[0],
//               GPTIMER_CCACT_01_ZACT_CCP_HIGH | GPTIMER_CCACT_01_CUACT_CCP_LOW,
//               GPTIMER_CCACT_01_CUACT_MASK | GPTIMER_CCACT_01_CDACT_MASK | GPTIMER_CCACT_01_CUACT_MASK |
//               GPTIMER_CCACT_01_LACT_MASK | GPTIMER_CCACT_01_ZACT_MASK);

    // CC1 action
//    update_reg(&TIMA1->COUNTERREGS.CCACT_01[1],
//               GPTIMER_CCACT_01_ZACT_CCP_HIGH | GPTIMER_CCACT_01_CUACT_CCP_LOW,
//               GPTIMER_CCACT_01_CUACT_MASK | GPTIMER_CCACT_01_CDACT_MASK | GPTIMER_CCACT_01_CUACT_MASK |
//               GPTIMER_CCACT_01_LACT_MASK | GPTIMER_CCACT_01_ZACT_MASK);

//
//    update_reg(&TIMA1->COUNTERREGS.CCCTL_01[0],
//               GPTIMER_CCCTL_01_COC_COMPARE, GPTIMER_CCCTL_01_COC_MASK);
//    update_reg(&TIMA1->COUNTERREGS.CCCTL_01[1],
//              GPTIMER_CCCTL_01_COC_COMPARE, GPTIMER_CCCTL_01_COC_MASK);
//
//    update_reg(&TIMA1->COUNTERREGS.CCCTL_01[0], GPTIMER_CCCTL_01_CCUPD_IMMEDIATELY,
//               GPTIMER_CCCTL_01_CCUPD_MASK);
//    update_reg(&TIMA1->COUNTERREGS.CCCTL_01[1], GPTIMER_CCCTL_01_CCUPD_IMMEDIATELY,
//               GPTIMER_CCCTL_01_CCUPD_MASK);

//    TIMA1->COUNTERREGS.CCCTL_01[0] = 0x0;
//    TIMA1->COUNTERREGS.CCCTL_01[1] = 0x0;

//    TIMA1->COUNTERREGS.OCTL_01[0] = GPTIMER_OCTL_01_CCPIV_LOW |
//            GPTIMER_OCTL_01_CCPOINV_NOINV | GPTIMER_OCTL_01_CCPO_FUNCVAL;
//    TIMA1->COUNTERREGS.OCTL_01[1] = GPTIMER_OCTL_01_CCPIV_LOW |
//            GPTIMER_OCTL_01_CCPOINV_NOINV | GPTIMER_OCTL_01_CCPO_FUNCVAL;
//    TIMA1->COUNTERREGS.CCCTL_01[0] = 0x0;
//    TIMA1->COUNTERREGS.CCCTL_01[1] = 0x0;

//    TIMA1->COUNTERREGS.IFCTL_01[0] = GPTIMER_IFCTL_01_INV_NOINVERT |
//            GPTIMER_IFCTL_01_ISEL_CCPX_INPUT;
//    TIMA1->COUNTERREGS.IFCTL_01[1] = GPTIMER_IFCTL_01_INV_NOINVERT |
//            GPTIMER_IFCTL_01_ISEL_CCPX_INPUT;
//    TIMA1->COUNTERREGS.IFCTL_01[0] = 0x0;
//    TIMA1->COUNTERREGS.IFCTL_01[1] = 0x0;

//    update_reg(&TIMA1->COUNTERREGS.CTRCTL,
//       GPTIMER_CTRCTL_REPEAT_REPEAT_1 | GPTIMER_CTRCTL_CM_UP |
//       GPTIMER_CTRCTL_CVAE_ZEROVAL | GPTIMER_CTRCTL_EN_DISABLED,
//       GPTIMER_CTRCTL_CM_MASK | GPTIMER_CTRCTL_REPEAT_MASK |
//       GPTIMER_CTRCTL_CVAE_MASK | GPTIMER_CTRCTL_EN_MASK);
//
    // ACOND (advance counter) is TIMCLK (0x0), ZCOND AND LCOND do nothing (0x0)
    // defaults are 7h, so they need to be written!
//    update_reg(&TIMA1->COUNTERREGS.CTRCTL,
//               GPTIMER_CTRCTL_CZC_CCCTL0_ZCOND | GPTIMER_CTRCTL_CAC_CCCTL0_ACOND |
//               GPTIMER_CTRCTL_CLC_CCCTL0_LCOND,
//               GPTIMER_CTRCTL_CZC_MASK | GPTIMER_CTRCTL_CAC_MASK |
//               GPTIMER_CTRCTL_CLC_MASK);



}



